
// ArrayBinaryTree:  Array Implementation of a Binary Tree
// Note that root node is at index 0

public class ArrayBinaryTree
{
	
	public static final int CAP = 1000;
	private Object S[];
	private int size;
	
	public ArrayBinaryTree()
	{
		S = new Object[CAP];
		for( int i = 0; i < CAP; i++ )
		{
			S[i] = null;
		}
		size = 0;
	}
	
	public int size()
	{
		return size;
	}
	
	public boolean isEmpty()
	{
		return (size == 0);
	}
	
	public boolean isInternal( int v )
	{
		return ( hasLeft(v) || hasRight(v) );
	}
	
	public boolean isExternal( int v )
	{
		return ( !isInternal(v) );
	}
	
	public boolean isRoot( int v )
	{
		return (v == 0);
	}
	
	public boolean hasLeft( int v )
	{
		checkOccupiedPosition( v );
		int leftPos = 2*v + 1;
		if ( leftPos < CAP && S[leftPos] != null )
			return true;
		else
			return false;
	}
	
	public boolean hasRight( int v )
	{
		checkOccupiedPosition( v );
		int rightPos = 2*v + 2;
		if ( rightPos < CAP && S[rightPos] != null )
			return true;
		else
			return false;
	}
	
	public int root()
	{
		return 0;
	}

	public int left( int v )
	{
		checkOccupiedPosition( v );
		checkPosition( 2*v + 1 );
		return 2*v + 1;
	}

	public int right( int v )
	{
		checkOccupiedPosition( v );
		checkPosition( 2*v + 2 );
		return 2*v + 2;
	}

	public int parent( int v ) throws NoSuchNodeException
	{
		checkPosition( v );
		if ( isRoot( v ) )
			throw new NoSuchNodeException();
		return (v-1)/2;
	}
	
	public Object getElement( int v )
	{
		checkOccupiedPosition( v );
		return S[v];
	}
	
	public int sibling( int v )
	{
		checkOccupiedPosition( v );
		int p = parent( v );
		int lc = left( p );
		if ( v == lc )
			return right(p);
		else
			return lc;
	}
	
	// replaces element at position v, returns previous contents
	public Object replace( int v, Object o ) throws NoSuchNodeException
	{
		checkPosition( v );
		Object temp = S[v];
		if ( temp == null )
			return new NoSuchNodeException();
		S[v] = o;
		return temp;
	}
	
	public int addRoot( Object e ) throws NodeExistsException
	{
		if ( S[0] != null )
			throw new NodeExistsException();
		S[0] = e;
		size++;
		return 0;
	}

	public int insertLeft( int v, Object e ) throws NodeExistsException
	{
		checkOccupiedPosition( v );
		if ( hasLeft( v ) )
			throw new NodeExistsException();
		int lc = left( v );
		S[lc] = e;
		size++;
		return lc;
	}

	public int insertRight( int v, Object e ) throws NodeExistsException
	{
		checkOccupiedPosition( v );
		if ( hasRight( v ) )
			throw new NodeExistsException();
		int rc = right( v );
		S[rc] = e;
		size++;
		return rc;
	}
	
	// remove a leaf node, return object originally at that node
	public Object remove( int v ) throws NoSuchNodeException, NotALeafException
	{
		checkPosition( v );
		if ( S[v] == null )
			throw new NoSuchNodeException();
		if ( isInternal( v ) )
			throw new NotALeafException();
		Object temp = S[v];
		S[v] = null;
		size--;
		return temp;
	}

	// convenience method used to check if int parameter is a valid position
	public void checkPosition( int v ) throws InvalidPositionException
	{
		if ( !( v >= 0 && v < CAP ) )
			throw new InvalidPositionException();
	}

	// convenience method used to check if int parameter is a valid position and is occupied
	public void checkOccupiedPosition( int v ) throws InvalidPositionException
	{
		if ( !isOccupied( v ) )
			throw new InvalidPositionException();
	}
	
	public boolean isOccupied( int v )
	{
		return ( v >= 0 && v < CAP  && S[v] != null );
	}

}
