
// LinkedBinaryTree:  Linked Implementation of a Binary Tree
// Requires BTNode class

public class LinkedBinaryTree
{
	private BTNode root;
	private int size;
	
	public LinkedBinaryTree()
	{
		root = null;
		size = 0;
	}
	
	public int size()
	{
		return size;
	}
	
	public boolean isEmpty()
	{
		return (size == 0);
	}
	
	public boolean isInternal( BTNode v )
	{
		return ( hasLeft(v) || hasRight(v) );
	}
	
	public boolean isExternal( BTNode v )
	{
		return ( !isInternal(v) );
	}
	
	public boolean isRoot( BTNode v )
	{
		checkPosition( v );
		return (v == root);
	}
	
	public boolean hasLeft( BTNode v )
	{
		checkPosition( v );
		return (v.getLeft() != null);
	}
	
	public boolean hasRight( BTNode v )
	{
		checkPosition( v );
		return (v.getRight() != null);
	}
	
	public BTNode root()
	{
		return root;
	}

	public BTNode left( BTNode v )
	{
		checkPosition( v );
		return v.getLeft();
	}

	public BTNode right( BTNode v )
	{
		checkPosition( v );
		return v.getRight();
	}

	public BTNode parent( BTNode v )
	{
		checkPosition( v );
		return v.getParent();
	}

	public Object getElement( BTNode v )
	{
		checkPosition( v );
		return v.getElement();
	}
	
	public BTNode sibling( BTNode v )
	{
		checkPosition( v );
		BTNode p = parent( v );
		BTNode lc = left( p );
		if ( v == lc )
			return right(p);
		else
			return lc;
	}
	
	// replaces element at position v, returns previous contents
	public Object replace( BTNode v, Object o ) throws NoSuchNodeException
	{
		if ( v == null )
			return new NoSuchNodeException();
		Object temp = v.getElement();
		v.setElement( o );
		return temp;
	}
	
	public BTNode addRoot( Object e ) throws NodeExistsException
	{
		if ( root != null )
			throw new NodeExistsException();
		root = new BTNode( e );
		size++;
		return root;
	}

	public BTNode insertLeft( BTNode v, Object e ) throws NodeExistsException
	{
		checkPosition( v );
		if ( hasLeft( v ) )
			throw new NodeExistsException();
		BTNode temp = new BTNode( e, null, null, v );
		v.setLeft( temp );
		size++;
		return temp;
	}

	public BTNode insertRight( BTNode v, Object e ) throws NodeExistsException
	{
		checkPosition( v );
		if ( hasRight( v ) )
			throw new NodeExistsException();
		BTNode temp = new BTNode( e, null, null, v );
		v.setRight( temp );
		size++;
		return temp;
	}
	
	// remove a leaf node, return object originally at that node
	public Object remove( BTNode v ) throws NoSuchNodeException, NotALeafException
	{
		if ( v == null )
			throw new NoSuchNodeException();
		if ( isInternal( v ) )
			throw new NotALeafException();
		Object temp = v.getElement();
		if (v.getParent().getLeft() == v)
			v.getParent().setLeft( null );
		else
			v.getParent().setRight( null );
		size--;
		return temp;
	}

	// convenience method used to check if BTNode parameter is a valid position
	public void checkPosition( BTNode v ) throws InvalidPositionException
	{
		if ( v == null )
			throw new InvalidPositionException();
	}

}